package org.greenstone.gatherer.collection;

import java.io.*;
import org.greenstone.gatherer.Gatherer;
import org.greenstone.gatherer.cdm.CommandTokenizer;
import org.greenstone.gatherer.msm.MSMUtils;
import org.greenstone.gatherer.util.Utility;
import org.w3c.dom.*;
/** This class provides access to a collection configuration file. This version accepts either a valid xml document or a historical collect.cfg file. */
public class CollectionConfiguration {

    private Element creator_element;
    private Element description_element;
    private Element maintainer_element;
    private Element name_element;
    private File file;
    private String creator;
    private String description;
    private String maintainer;
    private String name;

    public CollectionConfiguration(File file) {
	this.file = file;
	try {
	    String filename = file.getName().toLowerCase();
	    if(filename.endsWith(".xml")) {

	    }
	    else if(filename.endsWith(".cfg")) {
		FileReader fr = new FileReader(file);
		BufferedReader br = new BufferedReader(fr);
		String command = null;
		while((command = br.readLine()) != null) {
		    if(command.length() > 0) {
			// We have to test the end of command for the special character '\'. If found, remove it and append the next line, then repeat.
			while(command.trim().endsWith("\\")) {
			    command = command.substring(0, command.lastIndexOf("\\"));
			    String next_line = br.readLine();
			    if(next_line != null) {
				command = command + next_line;
			    }
			    next_line = null;
			}
			CommandTokenizer tokenizer = new CommandTokenizer(command);
			if(tokenizer.countTokens() >= 2) {
			    String temp = tokenizer.nextToken();
			    if(temp.equalsIgnoreCase("creator")) {
				creator = tokenizer.nextToken();
			    }
			    else if(temp.equalsIgnoreCase("maintainer")) {
				maintainer = tokenizer.nextToken();
			    }
			    else if(temp.equalsIgnoreCase("collectionmeta")) {
				temp = tokenizer.nextToken();
				if(temp.equalsIgnoreCase("collectionname")) {
				    temp = tokenizer.nextToken();
				    if(temp.startsWith("[") && temp.endsWith("]")) {
					String language = temp.substring(temp.indexOf("=") + 1, temp.length() - 1);
					if(name == null || language.equalsIgnoreCase(Gatherer.dictionary.getLanguage())) {
					    name = Utility.decodeGreenstone(tokenizer.nextToken());
					}
				    }
				    else if(name == null) {
					name = Utility.decodeGreenstone(temp);
				    }
				    if(name != null && name.startsWith("\"") && name.endsWith("\"")) {
					name = name.substring(1, name.length() - 1);
				    }
				}
				else if(temp.equalsIgnoreCase("collectionextra")) {
				    temp = tokenizer.nextToken();
				    ///ystem.err.println("Read: " + temp);
				    if(temp.startsWith("[") && temp.endsWith("]")) {
					String language = temp.substring(temp.indexOf("=") + 1, temp.length() - 1);
					///ystem.err.println("Try to match " + language + " to " + Gatherer.dictionary.getLanguage());
					if(description == null || language.equalsIgnoreCase(Gatherer.dictionary.getLanguage())) {
					    description = Utility.decodeGreenstone(tokenizer.nextToken());
					    ///ystem.err.println("Found language match, or first match: " + description);
					}
				    }
				    else if(description == null) {
					description = Utility.decodeGreenstone(temp);
					///ystem.err.println("Found first match: " + description);
				    }
				    if(description != null && description.startsWith("\"") && description.endsWith("\"")) {
					description = description.substring(1, description.length() - 1);
				    }
				}
			    }
			    temp = null;
			}
			tokenizer = null;
		    }
		}
		command = null;
		br.close();
		br = null;
		fr.close();
		fr = null;
	    }
				///ystem.err.println("Parsed collect.cfg");
				///ystem.err.println("name        = " + name);
				///ystem.err.println("creator     = " + creator);
				///ystem.err.println("maintainer  = " + maintainer);
				///ystem.err.println("description = " + description);
	}
	catch(Exception error) {
	    Gatherer.println("Error in CollectionConfiguration.<init>(): " + error);
	    Gatherer.printStackTrace(error);
	}
    }

    public String getCreator() {
	String result = "";
	if(creator_element != null) {
	    result = MSMUtils.getValue(creator_element);
	}
	else if(creator != null) {
	    result = creator;
	}
	return result;
    }

    public String getDescription() {
	String result = "";
	if(description_element != null) {
	    result = MSMUtils.getValue(description_element);
	}
	else if(description != null) {
	    result = description;
	}
	return result;
    }

	public File getFile() {
		return file;
	}

    public String getMaintainer() {
	String result = "";
	if(maintainer_element != null) {
	    result = MSMUtils.getValue(maintainer_element);
	}
	else if(maintainer != null) {
	    result = maintainer;
	}
	return result;
    }

    public String getName() {
	String result = "Error";
	if(name_element != null) {
	    result = MSMUtils.getValue(name_element);
	}
	else if(name != null) {
	    result = name;
	}
	return result;
    }

	/** Retrieve the short name for this collection which, given this current file is in <col_name>/etc/collect.cfg, is the name of this file's parent file's parent.
	* @return the short name of this collection as a String
	*/
	public String getShortName() {
		return file.getParentFile().getParentFile().getName();
	}

    public void setName(String name) {
	/** @todo */
    }

    /** Display the title for this collection. */
    public String toString() {
		return getName();
	}
}
