package org.greenstone.gsdl3.action;

import org.greenstone.gsdl3.core.ModuleInterface;
import org.greenstone.gsdl3.util.*;
// XML classes
import org.w3c.dom.Node; 
import org.w3c.dom.NodeList; 
import org.w3c.dom.Element; 
import org.w3c.dom.Document; 

// other java stuff
import java.io.File;
import java.util.Vector;
import java.util.HashMap;

/** base class for Actions */
abstract public class Action {

    /** the system set up variables */
    protected HashMap config_params = null;
    /** container Document to create XML Nodes */
    protected Document doc_=null; 
    /** a converter class to parse XML and create Docs */
    protected XMLConverter converter_=null;
    /** a reference to the message router that it must talk to to
     * get info. it may be a communicator acting as a proxy, but it 
     doesn't care about that */
    protected ModuleInterface mr_=null;

    public Action() {
	converter_ = new XMLConverter();
	doc_ = converter_.newDOM();
    }
    /** the config variables must be set before configure is called */
    public void setConfigParams(HashMap params) {
	this.config_params = params;
    }
    /** sets the message router */
    public void setMessageRouter(ModuleInterface m) {
       mr_ = m;
    }
    public void configure() {
	// does nothing yet
    }
    
    /** process takes an xml representation of cgi args
     * and returns the page of results - may be in html/xml/other
     * depending on the output att of the request */
    public String process(String xml_in) {
       
	Element message = converter_.getDOM(xml_in).getDocumentElement();
	
	Element result = process(message);
	return converter_.getString(result);
    }
    
    /** the main process method - must be implemented in subclass */
    abstract public Element process(Element xml_in);

//      protected Vector getMetadata(Element format) {
//  	NodeList nodes = format.getElementsByTagName("gsf:metadata");
//          Vector meta_names = new Vector();
//          for (int i=0; i<nodes.getLength(); i++) {
//              meta_names.add(((Element)nodes.item(i)).getAttribute("name"));
//          }
//          return meta_names;
//      }
}




